@@ -1,23 +1,21 @@
 #!/usr/bin/env python
-# -*- coding: utf-8 -*-
 
 # Author: Mathias Hauser
 # Date:
 
 import copy
-import warnings
+from collections import OrderedDict
 
 import numpy as np
-import six
 from shapely.geometry import MultiPolygon, Polygon
 
 from .formatting import _display
-from .mask import _mask_2D, _mask_3D
+from .mask import _inject_mask_docstring, _mask_2D, _mask_3D
 from .plot import _plot, _plot_regions
 from .utils import _is_180, _is_numeric, _maybe_to_dict, _sanitize_names_abbrevs
 
 
-class Regions(object):
+class Regions:
     """
     class for plotting regions and creating region masks
     """
@@ -49,8 +47,8 @@ class Regions(object):
         source : string, optional
             Source of the region definitions. Default: "".
 
-        Example
-        -------
+        Examples
+        --------
         Create your own ``Regions``::
 
             from regionmask import Regions
@@ -82,8 +80,6 @@ class Regions(object):
 
         """
 
-        super(Regions, self).__init__()
-
         if numbers is None:
             numbers = range(len(outlines))
 
@@ -95,9 +91,9 @@ class Regions(object):
         names = _sanitize_names_abbrevs(numbers, names, "Region")
         abbrevs = _sanitize_names_abbrevs(numbers, abbrevs, "r")
 
-        regions = dict()
+        regions = OrderedDict()
 
-        for n in numbers:
+        for n in sorted(numbers):
             regions[n] = _OneRegion(n, names[n], abbrevs[n], outlines[n])
 
         self.regions = regions
@@ -127,7 +123,9 @@ class Regions(object):
             return self.regions[key]
         else:
             # subsample the regions
-            regions = {k: self.regions[k] for k in key}
+            regions = OrderedDict()
+            for k in key:
+                regions[k] = self.regions[k]
             new_self = copy.copy(self)  # shallow copy
             new_self.regions = regions
             return new_self
@@ -147,13 +145,12 @@ class Regions(object):
         Returns
         -------
         mapped_key : int or list of int
-
         Raises a KeyError if the key does not exist.
 
         """
 
         # a single key
-        if isinstance(key, (int, np.integer, six.string_types)):
+        if isinstance(key, (int, np.integer, str)):
             key = self.region_ids[key]
         # a list of keys
         else:
@@ -230,8 +227,7 @@ class Regions(object):
 
     @property
     def bounds_global(self):
-        """global bounds over all regions (min_lon, min_lat, max_lon, max_lat)
-        """
+        """global bounds over all regions (min_lon, min_lat, max_lon, max_lat)"""
 
         bounds = self.bounds
 
@@ -244,8 +240,7 @@ class Regions(object):
 
     @property
     def lon_180(self):
-        """if the regions extend from -180 to 180
-        """
+        """if the regions extend from -180 to 180"""
         lon_min = self.bounds_global[0]
         lon_max = self.bounds_global[2]
 
@@ -253,8 +248,7 @@ class Regions(object):
 
     @property
     def lon_360(self):
-        """if the regions extend from 0 to 360
-        """
+        """if the regions extend from 0 to 360"""
         return not self.lon_180
 
     def _display(self, max_rows=10, max_width=None, max_colwidth=50):
@@ -275,8 +269,8 @@ class Regions(object):
         str or None
             Returns the result as a string.
 
-        Note
-        ----
+        Notes
+        -----
         Used as the repr.
 
         """
@@ -289,70 +283,23 @@ class Regions(object):
         lon_name="lon",
         lat_name="lat",
         method=None,
-        xarray=None,
         wrap_lon=None,
     ):
-        """
-        create a grid as mask of a set of regions for given lat/ lon grid
-
-        Parameters
-        ----------
-        lon_or_obj : object or array_like
-            Can either be a longitude array and then ``lat`` needs to be
-            given. Or an object where the longitude and latitude can be
-            retrived as: ``lon = lon_or_obj[lon_name]`` and
-            ``lat = lon_or_obj[lat_name]``
-        lat : array_like, optional
-            If 'lon_or_obj' is a longitude array, the latitude needs to be
-            specified here.
-        lon_name : str, optional
-            Name of longitude in 'lon_or_obj'. Default: 'lon'.
-        lat_name : str, optional
-            Name of latgitude in 'lon_or_obj'. Default: 'lat'
-        method : None | "rasterize" | "shapely" | "legacy"
-            Set method used to determine wether a gridpoint lies in a region.
-            If None (default) autoselects the method depending on the grid spacing.
-        xarray : None | bool, optional
-            Deprecated. If None or True returns an xarray DataArray, if False returns a
-            numpy ndarray. Default: None.
-        wrap_lon : None | bool | 180 | 360, optional
-            Whether to wrap the longitude around, should be inferred automatically.
-            If the regions and the provided longitude do not have the same
-            base (i.e. one is -180..180 and the other 0..360) one of them
-            must be wrapped. This can be done with wrap_lon.
-            If wrap_lon is None autodetects whether the longitude needs to be
-            wrapped. If wrap_lon is False, nothing is done. If wrap_lon is True,
-            longitude data is wrapped to 360 if its minimum is smaller
-            than 0 and wrapped to 180 if its maximum is larger than 180.
-
-        Returns
-        -------
-        mask : ndarray or xarray DataArray
-
-        References
-        ----------
-        See https://regionmask.readthedocs.io/en/stable/notebooks/method.html
-
-        """
-
-        if method == "legacy":
-            outlines = self.coords
-        else:
-            outlines = self.polygons
 
         return _mask_2D(
-            outlines=outlines,
-            regions_is_180=self.lon_180,
+            outlines=self.polygons,
+            lon_bounds=self.bounds_global[::2],
             numbers=self.numbers,
             lon_or_obj=lon_or_obj,
             lat=lat,
             lon_name=lon_name,
             lat_name=lat_name,
             method=method,
-            xarray=xarray,
             wrap_lon=wrap_lon,
         )
 
+    mask.__doc__ = _inject_mask_docstring(is_3D=False, gp_method=False)
+
     def mask_3D(
         self,
         lon_or_obj,
@@ -363,52 +310,10 @@ class Regions(object):
         method=None,
         wrap_lon=None,
     ):
-        """
-        create a 3D boolean mask of a set of regions for the given lat/ lon grid
-
-        Parameters
-        ----------
-        lon_or_obj : object or array_like
-            Can either be a longitude array and then ``lat`` needs to be
-            given. Or an object where the longitude and latitude can be
-            retrived as: ``lon = lon_or_obj[lon_name]`` and
-            ``lat = lon_or_obj[lat_name]``
-        lat : array_like, optional
-            If ``lon_or_obj`` is a longitude array, the latitude needs to be
-            specified here.
-        drop : boolean, optional
-            If True (default) drops slices where all elements are False (i.e no
-            gridpoints are contained in a region). If False returns one slice per
-            region.
-        lon_name : str, optional
-            Name of longitude in 'lon_or_obj'. Default: "lon".
-        lat_name : str, optional
-            Name of latgitude in 'lon_or_obj'. Default: "lat"
-        method : None | "rasterize" | "shapely", optional
-            Set method used to determine wether a gridpoint lies in a region.
-            If None (default) autoselects the method depending on the grid spacing.
-        wrap_lon : None | bool | 180 | 360, optional
-            Whether to wrap the longitude around, should be inferred automatically.
-            If the regions and the provided longitude do not have the same
-            base (i.e. one is -180..180 and the other 0..360) one of them
-            must be wrapped. This can be done with wrap_lon.
-            If wrap_lon is None autodetects whether the longitude needs to be
-            wrapped. If wrap_lon is False, nothing is done. If wrap_lon is True,
-            longitude data is wrapped to 360 if its minimum is smaller
-            than 0 and wrapped to 180 if its maximum is larger than 180.
-
-        Returns
-        -------
-        mask_3D : boolean xarray.DataArray
-
-        """
-
-        if method == "legacy":
-            raise ValueError("method 'legacy' not supported in 'mask_3D'")
 
         mask_3D = _mask_3D(
             outlines=self.polygons,
-            regions_is_180=self.lon_180,
+            lon_bounds=self.bounds_global[::2],
             numbers=self.numbers,
             lon_or_obj=lon_or_obj,
             lat=lat,
@@ -429,105 +334,22 @@ class Regions(object):
 
         return mask_3D
 
+    mask_3D.__doc__ = _inject_mask_docstring(is_3D=True, gp_method=False)
+
 
 # add the plotting methods
 Regions.plot = _plot
 Regions.plot_regions = _plot_regions
 
 
-class Regions_cls(Regions):
-    def __init__(
-        self, name, numbers, names, abbrevs, outlines, centroids=None, source=""
-    ):
-        """
-        Parameters
-        ----------
-        name : string
-            Name of the collection of regions.
-        numbers : list of int
-            List of numerical indices for every region.
-        names : dict of string
-            Long name of each region. Must be accessible as names[number].
-        abbrevs : dict of string
-            List of abbreviations of each region. Must be accessible as
-            abbrevs[number].
-        outlines : List of Nx2 float array of vertices, Polygon, MultiPolygon
-            List of coordinates/ outline of the region as shapely
-            Polygon/ MultiPolygon or list. Must be accessible as
-            outlines[number].
-        centroids : list of 1x2 iterable, optional.
-            Center of mass of the regions. If not provided is calculated
-            as (Multi)Polygon.centroid. Position of the label on map plots.
-        source : string, optional
-            Source of the region definitions. Default: ''.
-
-        Example
-        -------
-        Unsing numpy style outlines::
-
-            name = 'Example'
-            numbers = [0, 1]
-            names = ['Unit Square1', 'Unit Square2']
-            abbrevs = ['uSq1', 'uSq2']
-
-            outl1 = ((0, 0), (0, 1), (1, 1.), (1, 0))
-            outl2 = ((0, 1), (0, 2), (1, 2.), (1, 1))
-            outlines = [outl1, outl2]
-
-            r = Regions_cls(name, numbers, names, abbrevs, outlines)
-
-        Using shapely Polygons:
-
-            from shapely.geometry import Polygon
-
-            numbers = [1, 2]
-            names = {1:'Unit Square1', 2: 'Unit Square2'}
-            abbrevs = {1:'uSq1', 2:'uSq2'}
-            poly = {1: Polygon(outl1), 2: Polygon(outl2)}
-
-            r = Regions_cls(name, numbers, names, abbrevs, poly)
-        """
-
-        msg = (
-            "Using 'Regions_cls' is deprecated, please use 'Regions' instead."
-            " Please note that the call signature is different."
-        )
-        warnings.warn(msg, FutureWarning, stacklevel=3)
-
-        super(Regions_cls, self).__init__(
-            outlines=outlines,
-            numbers=numbers,
-            names=names,
-            abbrevs=abbrevs,
-            name=name,
-            source=source,
-        )
-
-        if centroids is not None:
-            msg = (
-                "Specifying 'centroids' is deprecated and the keyword will be "
-                "removed in the next version."
-            )
-            warnings.warn(msg, FutureWarning, stacklevel=3)
-
-            numbers = self.numbers
-
-            centroids = _maybe_to_dict(numbers, centroids)
-
-            for number in numbers:
-                self[number].centroid = centroids[number]
-
-
 # =============================================================================
 
 
-class _OneRegion(object):
-    """a single Region, used as member of 'Regions'
-    """
+class _OneRegion:
+    """a single Region, used as member of 'Regions'"""
 
     def __init__(self, number, name, abbrev, outline, centroid=None):
         """
-
         Parameters
         ----------
         number : int
@@ -543,9 +365,8 @@ class _OneRegion(object):
             Center of mass of this region. If not provided is calculated
             as (Multi)Polygon.centroid. Position of the label on map plots.
 
-        Example
-        -------
-
+        Examples
+        --------
         ``_OneRegion`` can be created with numpy-style outlines::
 
             outl = ((0, 0), (0, 1), (1, 1.), (1, 0))
@@ -559,8 +380,6 @@ class _OneRegion(object):
             r = _OneRegion(1, 'Unit Square', 'USq', poly, centroid=[0.5, 0.75])
         """
 
-        super(_OneRegion, self).__init__()
-
         self.number = number
         self.name = name
         self.abbrev = abbrev
@@ -643,12 +462,3 @@ class _OneRegion(object):
         if self._bounds is None:
             self._bounds = self.polygon.bounds
         return self._bounds
-
-
-class Region_cls(_OneRegion):
-    def __init__(self, number, name, abbrev, outline, centroid=None):
-
-        msg = "Using 'Region_cls' is deprecated, please use '_OneRegion' instead."
-        warnings.warn(msg, FutureWarning, stacklevel=3)
-
-        super(Region_cls, self).__init__(number, name, abbrev, outline, centroid)
